@@ -1,25 +1,25 @@
-# (C) Copyright IBM Corp. 2020.
+#  (C) Copyright IBM Corp. 2021.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#       http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
 
 import copy
 from typing import List, Union
 
 from ibm_watson_machine_learning.preprocessing import DataJoinGraph
 from ibm_watson_machine_learning.utils.autoai.enums import (
-    TShirtSize, ClassificationAlgorithms, RegressionAlgorithms, ForecastingAlgorithms, PredictionType, Metrics,\
+    TShirtSize, ClassificationAlgorithms, RegressionAlgorithms, ForecastingAlgorithms, PredictionType, Metrics, \
     Transformers, DataConnectionTypes, PipelineTypes, PositiveLabelClass, ClassificationAlgorithmsCP4D,
-    RegressionAlgorithmsCP4D, ForecastingAlgorithmsCP4D, SamplingTypes)
+    RegressionAlgorithmsCP4D, ForecastingAlgorithmsCP4D, SamplingTypes, ImputationStrategy)
 from ibm_watson_machine_learning.utils.autoai.errors import LocalInstanceButRemoteParameter, MissingPositiveLabel, \
     NonForecastPredictionColumnMissing, ForecastPredictionColumnsMissing, ForecastingCannotBeRunAsLocalScenario, \
     TSNotSupported, ParamOutOfRange
@@ -168,12 +168,17 @@ class AutoAI(BaseExperiment):
                   positive_label: str = None,
                   data_join_only: bool = False,
                   drop_duplicates: bool = True,
-                  # fairness_info: dict = None, #TODO: Uncomment to add fairness support
                   text_processing: bool = None,
                   word2vec_feature_number: int = None,
                   daub_give_priority_to_runtime: float = None,
+                  fairness_info: dict = None,
                   # sampling_type: 'SamplingTypes' = None, #TODO: Uncomment to add sampling_type support
-                  n_parallel_data_connections: int =None,
+                  n_parallel_data_connections: int = None,
+                  categorical_imputation_strategy: ImputationStrategy = None,
+                  numerical_imputation_strategy: ImputationStrategy = None,
+                  numerical_imputation_value: float = None,
+                  imputation_threshold: float = None,
+                  retrain_on_holdout: int = True,
                   **kwargs) -> Union['RemoteAutoPipelines', 'LocalAutoPipelines']:
         """
         Initialize an AutoAi optimizer.
@@ -285,10 +290,22 @@ class AutoAI(BaseExperiment):
             If set to 0.0 only score is used. If set to 1 equally score and runtime are used.
             If set to value higher than 1 the runtime gets higher importance over score.
 
+        fairness_info: dict, optional
+            Dictionary that  specifies metadata needed for measuring fairness. It contains three key values:
+            favorable_labels, unfavorable_labels and protected_attributes. The favorable_labels attribute indicates that
+            when the class column contains one of the value from list, that is considered a positive outcome.
+            The unfavorable_labels is oposite to the favorable_labels and is obigatory for regression learning type.
+            A protected attribute is a list of features that partition the population into groups whose outcome should
+            have parity, if protected attribute is empty list then automatic detection of protected attributes will be run.
+            If passed fairness metric will be calculated.
+
         n_parallel_data_connections: int, optional
             Number of maximum parallel connection to data source.
             Supported only for IBM Cloud Pak® for Data 4.0.1 and above.
 
+        retrain_on_holdout: bool, optional
+            If True final pipelines will be train also on holdout data.
+
         Returns
         -------
         RemoteAutoPipelines or LocalAutoPipelines, depends on how you initialize the AutoAI object.
@@ -298,6 +315,14 @@ class AutoAI(BaseExperiment):
         >>> from ibm_watson_machine_learning.experiment import AutoAI
         >>> experiment = AutoAI(...)
         >>>
+        >>> fairness_info = {
+        >>>            "protected_attributes": [
+        >>>                {"feature": "Sex", "reference_group": ['male'], "monitored_group": ['female']},
+        >>>                {"feature": "Age", "reference_group": [[50,60]], "monitored_group": [[18, 49]]}
+        >>>            ],
+        >>>            "favorable_labels": ["No Risk"],
+        >>>            "unfavorable_labels": ["Risk"],
+        >>>            }
         >>>
         >>> optimizer = experiment.optimizer(
         >>>        name="name of the optimizer.",
@@ -307,6 +332,7 @@ class AutoAI(BaseExperiment):
         >>>        desc="Some description.",
         >>>        holdout_size=0.1,
         >>>        max_num_daub_ensembles=1,
+        >>>        fairness_info= fairness_info,
         >>>        cognito_transform_names=[AutoAI.Transformers.SUM,AutoAI.Transformers.MAX],
         >>>        train_sample_rows_test_size=1,
         >>>        include_only_estimators=[AutoAI.ClassificationAlgorithms.LGBM, AutoAI.ClassificationAlgorithms.XGB],
@@ -322,24 +348,6 @@ class AutoAI(BaseExperiment):
         >>>    )
         """
 
-        # # TODO: Pass the fairness info description and example when adding fairness support and
-        #
-        # """
-        # fairness_info: dict, optional
-        #     Dictionary that  specifies metadata needed for measuring fairness. It contains two key values:
-        #     favorable_labels and protected_attributes. The favorable_labels attribute indicates that when the class
-        #     column contains one of the value from list, that is considered a positive outcome.
-        #     A protected attribute is a feature that partitions the population into groups whose outcome should have parity.
-        #     If passed fairness metric will be calculated.
-        # >>> fairness_info = {
-        # >>>        "protected_attributes": [
-        # >>>            {"feature": "Sex", "privileged_groups": ['female']},
-        # >>>             {"feature": "Age", "privileged_groups": [[20, 40], [60, 90]]}
-        # >>>        ],
-        # >>>        "favorable_labels": ["No Risk"]
-        # >>>    }
-        # """
-
         # TODO: Pass the sampling_type doc description, when adding support for that
         # sampling_type: str, optional
         #     Type of sampling data for training. One of SamplingTypes enum values.
@@ -353,6 +361,10 @@ class AutoAI(BaseExperiment):
             raise TSNotSupported()
 
         if prediction_type == PredictionType.FORECASTING:
+            # TODO uncomment when TS will support imputation
+            # if not numerical_imputation_strategy:
+            #     numerical_imputation_strategy = ImputationStrategy.BEST_OF_DEFAULT_IMPUTERS
+
             if prediction_column is not None or prediction_columns is None:
                 raise ForecastPredictionColumnsMissing()
         else:
@@ -376,6 +388,7 @@ class AutoAI(BaseExperiment):
             daub_include_only_estimators=daub_include_only_estimators,
             include_only_estimators=include_only_estimators,
             cognito_transform_names=cognito_transform_names,
+            imputation_strategies=list(filter(None, [categorical_imputation_strategy, numerical_imputation_strategy])),
             scoring=scoring,
             t_shirt_size=kwargs.get("t_shirt_size", TShirtSize.M),
             is_cpd=self._workspace.wml_client.ICP
@@ -476,9 +489,14 @@ class AutoAI(BaseExperiment):
                 notebooks=kwargs.get('notebooks', True),
                 autoai_pod_version=kwargs.get('autoai_pod_version', None),
                 obm_pod_version=kwargs.get('obm_pod_version', None),
-                # fairness_info=fairness_info, #TODO: Uncomment to add fairness support
+                fairness_info=fairness_info,
                 # sampling_type=sampling_type, #TODO: Uncomment to add sampling_type support
                 n_parallel_data_connections=n_parallel_data_connections,
+                categorical_imputation_strategy=categorical_imputation_strategy,
+                numerical_imputation_strategy=numerical_imputation_strategy,
+                numerical_imputation_value=numerical_imputation_value,
+                imputation_threshold=imputation_threshold,
+                retrain_on_holdout=retrain_on_holdout,
                 **reduced_kwargs
             )
             optimizer._workspace = self._workspace
