@@ -1,16 +1,16 @@
-# (C) Copyright IBM Corp. 2020.
+#  (C) Copyright IBM Corp. 2021.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#       http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
 
 from typing import TYPE_CHECKING, List, Dict, Tuple
 
@@ -23,15 +23,18 @@ import logging
 
 from lomond import WebSocket
 from ibm_watson_machine_learning.utils.autoai.enums import (
-    PredictionType, RegressionAlgorithms, ClassificationAlgorithms, RunStateTypes, DataConnectionTypes)
+    PredictionType, RegressionAlgorithms, ClassificationAlgorithms, RunStateTypes, DataConnectionTypes,
+    ImputationStrategy)
 from ibm_watson_machine_learning.utils.autoai.errors import PipelineNotLoaded, FitNeeded, InvalidPredictionType, \
     AutoAIComputeError, ForecastingUnsupportedOperation, NoAvailableMetrics, LibraryNotCompatible, \
-    CannotConnectToWebsocket
+    CannotConnectToWebsocket, NumericalImputationStrategyValueMisused, InvalidImputationParameterNonTS, \
+    InvalidImputationParameterTS
 from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
 from ibm_watson_machine_learning.utils.autoai.progress_bar import ProgressBar
 from ibm_watson_machine_learning.utils.autoai.utils import (
     fetch_pipelines, is_ipython, ProgressGenerator, check_dependencies_versions, try_import_autoai_libs,
-    create_summary, prepare_auto_ai_model_to_publish_normal_scenario, get_sw_spec_and_type_based_on_sklearn)
+    create_summary, prepare_auto_ai_model_to_publish_normal_scenario, get_sw_spec_and_type_based_on_sklearn,
+    get_values_for_imputation_strategy, try_import_autoai_ts_libs)
 from .base_engine import BaseEngine
 from ibm_watson_machine_learning.helpers.connections import DataConnection
 
@@ -98,7 +101,7 @@ class WMLEngine(BaseEngine):
                                 'cognito_transform_names':
                                     self._auto_pipelines_parameters.get('cognito_transform_names'),
                                 'run_cognito_flag': True,
-                                'holdout_param': self._auto_pipelines_parameters.get('holdout_size')
+                                'holdout_param': self._auto_pipelines_parameters.get('holdout_size'),
                             }
                         },
                         'runtime_ref': 'autoai',
@@ -206,6 +209,9 @@ class WMLEngine(BaseEngine):
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
                     'gap_len'] = self._auto_pipelines_parameters['backtest_gap_length']
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'compute_pipeline_notebooks_flag'] = self._auto_pipelines_parameters.get('notebooks', False)
         else:
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                 'pipelines'][0]['nodes'][0]['parameters']['optimization'][
@@ -217,6 +223,10 @@ class WMLEngine(BaseEngine):
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
                     'scorer_for_ranking'] = self._auto_pipelines_parameters.get('scoring')
+            if 'retrain_on_holdout' in self._auto_pipelines_parameters:
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization'][
+                    'retrain_on_holdout'] = self._auto_pipelines_parameters.get('retrain_on_holdout')
 
         if self._20_class_limit_removal_test:
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['pipelines'][0]['nodes'][0]['parameters'].update({
@@ -310,6 +320,47 @@ class WMLEngine(BaseEngine):
                 'test_encoding'] = self._auto_pipelines_parameters.get('test_data_encoding')
         # --- end note
 
+        if self._auto_pipelines_parameters.get('prediction_type') == PredictionType.FORECASTING:
+            if self._auto_pipelines_parameters.get('categorical_imputation_strategy'):
+                raise InvalidImputationParameterTS()
+
+            if self._auto_pipelines_parameters.get('numerical_imputation_strategy'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization']\
+                    .update(get_values_for_imputation_strategy(
+                    self._auto_pipelines_parameters.get('numerical_imputation_strategy'),
+                    self._auto_pipelines_parameters.get('prediction_type')))
+
+                if self._auto_pipelines_parameters.get('numerical_imputation_value') is not None:
+                    if self._auto_pipelines_parameters.get('numerical_imputation_strategy') == ImputationStrategy.VALUE:
+                        self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                            'pipelines'][0]['nodes'][0]['parameters']['optimization']['imputer_fill_value'] = \
+                            self._auto_pipelines_parameters.get('numerical_imputation_value')
+                    else:
+                        raise NumericalImputationStrategyValueMisused()
+
+                if self._auto_pipelines_parameters.get('imputation_threshold'):
+                    self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                        'pipelines'][0]['nodes'][0]['parameters']['optimization']['imputation_threshold'] = \
+                        self._auto_pipelines_parameters.get('imputation_threshold')
+        else:
+            if self._auto_pipelines_parameters.get('numerical_imputation_value') is not None or self._auto_pipelines_parameters.get('imputation_threshold'):
+                raise InvalidImputationParameterNonTS()
+
+            if self._auto_pipelines_parameters.get('categorical_imputation_strategy'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization']['preprocessor_cat_imp_strategy'] = \
+                    get_values_for_imputation_strategy(
+                        self._auto_pipelines_parameters.get('categorical_imputation_strategy'),
+                        self._auto_pipelines_parameters.get('prediction_type'))
+
+            if self._auto_pipelines_parameters.get('numerical_imputation_strategy'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization']['preprocessor_num_imp_strategy'] = \
+                    get_values_for_imputation_strategy(
+                        self._auto_pipelines_parameters.get('numerical_imputation_strategy'),
+                        self._auto_pipelines_parameters.get('prediction_type'))
+
         # note: KB part: send version autoai_pod_version if set in pipeline details details (See supported formats)
         if not self._wml_client.ICP and self._auto_pipelines_parameters['autoai_pod_version'] is not None:
             self._wml_pipeline_metadata[
@@ -429,7 +480,7 @@ class WMLEngine(BaseEngine):
 
         if test_data_references is not None:
             self._wml_training_metadata[
-                self._wml_client.training.ConfigurationMetaNames.TEST_DATA_REFERENCE
+                self._wml_client.training.ConfigurationMetaNames.TEST_DATA_REFERENCES
             ] = [connection._to_dict() for connection in test_data_references]
 
         # note: Delete unnecessary keys for connection_asset in training_data_reference
@@ -635,10 +686,15 @@ class WMLEngine(BaseEngine):
 
         return self._wml_client.training.get_status(training_uid=self._current_run_id).get('state')
 
-    def get_run_details(self) -> dict:
+    def get_run_details(self, include_metrics: bool = False) -> dict:
         """
         Get fit/run details.
 
+        Parameters
+        ----------
+        include_metrics: bool, optional
+            Indicates to include metrics in the training details output. Default False.
+
         Returns
         -------
         Dictionary with AutoPipelineOptimizer fit/run details.
@@ -649,6 +705,9 @@ class WMLEngine(BaseEngine):
 
         details = self._wml_client.training.get_details(training_uid=self._current_run_id)
 
+        if include_metrics:
+            return details
+
         if details['entity']['status'].get('metrics', False):
             del details['entity']['status']['metrics']
             return details
@@ -713,6 +772,10 @@ class WMLEngine(BaseEngine):
                 pipeline_parameters['composition_steps'] = pipeline['context']['intermediate_model'][
                     'composition_steps']
                 pipeline_parameters['pipeline_nodes'] = pipeline['context']['intermediate_model']['pipeline_nodes']
+
+                if 'fairness' in pipeline['context']:
+                    pipeline_parameters['fairness_details'] = pipeline['context'].get('fairness')
+
                 if 'timeseries' not in pipeline['context']:
                     pipeline_parameters['ml_metrics'] = self._get_metrics(pipeline)
                 else:
@@ -914,7 +977,7 @@ class WMLEngine(BaseEngine):
         # --- end note
 
         if 'timeseries' in run_params['entity']['status']['metrics'][0]['context']:
-            raise ForecastingUnsupportedOperation()
+            try_import_autoai_ts_libs()
 
         try:
             pipelines, check_lale = fetch_pipelines(
