@@ -1,21 +1,21 @@
-# (C) Copyright IBM Corp. 2020.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 __all__ = [
     "AutoPipelinesRuns"
 ]
 
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 from copy import deepcopy
 from typing import List, Dict, Union, Optional
 
@@ -204,7 +204,7 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
                 'holdout_size': kb_parameters['holdout_param'],
                 'max_num_daub_ensembles': kb_parameters['max_num_daub_ensembles'],
                 't_shirt_size': kb_wml_data['hardware_spec']['id'],
-                'include_only_estimators': kb_parameters.get('include_only_estimators'),
+                'include_only_estimators': kb_parameters.get('daub_include_only_estimators'),
                 'cognito_transform_names': kb_parameters.get('cognito_transform_names'),
                 'train_sample_rows_test_size': kb_parameters.get('train_sample_rows_test_size'),
                 'text_processing': kb_parameters.get('text_processing_flag'),
@@ -214,7 +214,8 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
                 'drop_duplicates': drop_duplicates,
                 'csv_separator': csv_separator,
                 'excel_sheet': excel_sheet,
-                'encoding': encoding
+                'encoding': encoding,
+                'retrain_on_holdout': kb_parameters.get('retrain_on_holdout')
             }
 
             if kb_parameters.get('train_sample_rows_test_size'):
@@ -223,11 +224,14 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
                 params['scoring'] = kb_parameters['scorer_for_ranking']
             if kb_parameters.get('text_processing_options') and kb_parameters['text_processing_options'].get('word2vec'):
                 params['word2vec_feature_number'] = kb_parameters['text_processing_options']['word2vec'].get('output_dim')
+            if kb_parameters.get('fairness_info'):
+                params['fairness_info'] = kb_parameters.get('fairness_info')
             if kb_parameters['learning_type'] == 'timeseries':
                 params['backtest_num'] = kb_parameters.get('backtest_num')
                 params['lookback_window'] = kb_parameters.get('lookback_window')
                 params['forecast_window'] = kb_parameters.get('forecast_window')
                 params['backtest_gap_length'] = kb_parameters.get('backtest_gap_length')
+                params['include_only_estimators'] = kb_parameters.get('include_only_estimators')
         else:
             params = {
                 'name': name,
