@@ -1,17 +1,3 @@
-# (C) Copyright IBM Corp. 2020.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 __all__ = [
     "DataConnection",
     "S3Connection",
@@ -31,6 +17,20 @@ __all__ = [
     "ContainerLocation"
 ]
 
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import io
 import os
 import uuid
@@ -336,6 +336,15 @@ class DataConnection(BaseDataConnection):
                     )
                 )
 
+            elif _dict['location'].get('path') is not None and _dict['location'].get('bucket'):
+                data_connection: 'DataConnection' = cls(
+                    connection_asset_id=_dict['connection']['id'],
+                    location=S3Location(
+                        bucket=_dict['location']['bucket'],
+                        path=_dict['location']['path']
+                    )
+                )
+
             elif _dict['location'].get('schema_name') and _dict['location'].get('table_name'):
                 data_connection: 'DataConnection' = cls(
                     connection_asset_id=_dict['connection']['id'],
@@ -365,7 +374,8 @@ class DataConnection(BaseDataConnection):
 
     def _recreate_holdout(
             self,
-            data: 'DataFrame'
+            data: 'DataFrame',
+            with_holdout_split: bool = True
     ) -> Union[Tuple['DataFrame', 'DataFrame'], Tuple['DataFrame', 'DataFrame', 'DataFrame', 'DataFrame']]:
         """This method tries to recreate holdout data."""
         try_import_autoai_libs(minimum_version='1.12.14')
@@ -375,13 +385,13 @@ class DataConnection(BaseDataConnection):
         data.replace([np.inf, -np.inf], np.nan, inplace=True)
         data.drop_duplicates(inplace=True)
         data.dropna(subset=[self.auto_pipeline_params['prediction_column']], inplace=True)
-        dfy = DataFrame(data[self.auto_pipeline_params['prediction_column']])
+        dfy = data[self.auto_pipeline_params['prediction_column']]
         data.drop(columns=[self.auto_pipeline_params['prediction_column']], inplace=True)
 
-        y_column = dfy.columns
+        y_column = [self.auto_pipeline_params['prediction_column']]
         X_columns = data.columns
 
-        if self._test_data:
+        if self._test_data or not with_holdout_split:
             return data, dfy
 
         else:
@@ -390,7 +400,7 @@ class DataConnection(BaseDataConnection):
             from autoai_libs.utils.holdout_utils import numpy_remove_missing_target_rows
             # Remove (and save) the rows of X and y for which the target variable has missing values
             data, dfy, _, _, _, _ = numpy_remove_missing_target_rows(
-                y=dfy.values[:, 0], X=data.values
+                y=dfy, X=data
             )
             #   End of REMOVE MISSING ROWS    #
             ###################################
@@ -442,6 +452,7 @@ class DataConnection(BaseDataConnection):
              csv_separator: str = ',',
              excel_sheet: Union[str, int] = 0,
              encoding: Optional[str] = 'utf-8',
+             raw: Optional[bool] = False,
              **kwargs) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
         """
         Download dataset stored in remote data storage.
@@ -460,6 +471,10 @@ class DataConnection(BaseDataConnection):
         encoding: str, optional
             Encoding type of the CSV
 
+        raw: bool, optional
+            Default False. If False there will be applied simple data preprocessing (the same as in the backend).
+            If True, data will be not preprocessed.
+
         Returns
         -------
         pandas.DataFrame contains dataset from remote data storage : Xy_train
@@ -501,9 +516,9 @@ class DataConnection(BaseDataConnection):
             self.auto_pipeline_params['train_sample_rows_test_size'] = experiment_metadata.get(
                 'train_sample_rows_test_size')
             self.auto_pipeline_params['prediction_column'] = experiment_metadata['prediction_column']
-            self.auto_pipeline_params['holdout_size'] = experiment_metadata['holdout_size']
+            self.auto_pipeline_params['holdout_size'] = experiment_metadata.get('holdout_size')
             self.auto_pipeline_params['prediction_type'] = experiment_metadata['prediction_type']
-            self.auto_pipeline_params['fairness_info'] = experiment_metadata.get('fairness_info', None)
+            self.auto_pipeline_params['fairness_info'] = experiment_metadata.get('fairness_info')
 
             if self._test_data:
                 csv_separator = experiment_metadata.get('test_data_csv_separator', csv_separator)
@@ -625,6 +640,7 @@ class DataConnection(BaseDataConnection):
                             raise ConnectionError(
                                 f"Unable to access data object in cloud object storage with credentials supplied. "
                                 f"Error: {cos_access_exception}")
+
                     else:
                         data = self._download_data_from_nfs_connection()
 
@@ -653,16 +669,18 @@ class DataConnection(BaseDataConnection):
         if isinstance(data, DataFrame) and 'Unnamed: 0' in data.columns.tolist():
             data.drop(['Unnamed: 0'], axis=1, inplace=True)
 
-        if self._test_data:  # when this data connection is a test / holdout one
-            test_X, test_y = self._recreate_holdout(data=data)
-            return test_X, test_y
-
-        elif with_holdout_split:  # when this connection is training one
-            X_train, X_holdout, y_train, y_holdout = self._recreate_holdout(data=data)
-            return X_train, X_holdout, y_train, y_holdout
+        if raw or self.auto_pipeline_params.get('prediction_column') is None:
+            return data
 
         else:
-            return data
+            if with_holdout_split:  # when this connection is training one
+                X_train, X_holdout, y_train, y_holdout = self._recreate_holdout(data=data)
+                return X_train, X_holdout, y_train, y_holdout
+
+            else:  # when this data connection is a test / holdout one
+                test_X, test_y = self._recreate_holdout(data=data, with_holdout_split=False)
+                test_X[self.auto_pipeline_params.get('prediction_column', 'prediction_column')] = test_y
+                return test_X   # return one dataframe
 
     def write(self, data: Union[str, 'DataFrame'], remote_name: str = None, **kwargs) -> None:
         """
