@@ -1,21 +1,22 @@
-# (C) Copyright IBM Corp. 2020.
+#  (C) Copyright IBM Corp. 2021.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#       http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
 
 from __future__ import print_function
 import requests
 from ibm_watson_machine_learning.utils import STR_TYPE, STR_TYPE_NAME, docstring_parameter, meta_props_str_conv
 from ibm_watson_machine_learning.metanames import RemoteTrainingSystemMetaNames
+from ibm_watson_machine_learning.party_wrapper import Party
 import os
 import json
 from ibm_watson_machine_learning.wml_client_error import WMLClientError, UnexpectedType, ApiRequestFailure
@@ -62,7 +63,7 @@ class RemoteTrainingSystem(WMLResource):
         WMLResource._chk_and_block_create_update_for_python36(self)
         if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Not supported in this release")
-        
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(meta_props, u'meta_props', dict, True)
@@ -162,7 +163,7 @@ class RemoteTrainingSystem(WMLResource):
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_details(self, remote_training_system_id=None, limit=None, asynchronous=False, get_all=False):
         """
-            Get metadata of the given remote training system. if no remote_training_systems_id is specified all 
+            Get metadata of the given remote training system. if no remote_training_systems_id is specified all
             remote training systems metadata is returned.
 
            **Parameters**
@@ -199,9 +200,9 @@ class RemoteTrainingSystem(WMLResource):
          """
         if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Not supported in this release")
-        
+
         self._client._check_if_either_is_set()
-        
+
         RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_systems_id', STR_TYPE, False)
         RemoteTrainingSystem._validate_type(limit, u'limit', int, False)
 
@@ -212,7 +213,7 @@ class RemoteTrainingSystem(WMLResource):
                                               _async=asynchronous, _all=get_all)
         else:
             return self._get_artifact_details(href, remote_training_system_id, limit, 'remote_training_systems')
-    
+
     def list(self, limit=None):
         """
             List stored remote training systems. If limit is set to None there will be only first 50 records shown.
@@ -396,7 +397,7 @@ class RemoteTrainingSystem(WMLResource):
         """
         if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Not supported in this release")
-        
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, True)
@@ -414,3 +415,74 @@ class RemoteTrainingSystem(WMLResource):
              m[u'metadata'][u'created_at']) for m in resources]
 
         self._list(values, [u'ID', u'rev', u'NAME', u'CREATED'], limit, _DEFAULT_LIST_LENGTH)
+
+    def _validate_party_input(self, party_metadata):
+
+        if "data_handler" not in party_metadata:
+            raise WMLClientError("Its mandatory to provide 'DATA_HANDLER' in meta_props. Example: "
+                                 "client.remote_training_systems.ConfigurationMetaNames.DATA_HANDLER")
+
+
+
+    def create_party(self, remote_training_system_id, party_metadata):
+        """
+        Create a party object using the specified remote training system id and the party metadata.
+
+        **Parameters**
+
+        .. important::
+           #. **remote_training_system_id**:  remote training system identifier\n
+              **type**: str\n
+           #. **party_metadata**:  the party configuration\n
+              **type**: dict\n
+
+        **Output**
+
+        .. important::
+
+            **returns**: A party object with the specified rts_id and configuration\n
+            **return type**: Party\n
+
+        **Example**
+         >>> party_metadata = {
+         >>>     wml_client.remote_training_systems.ConfigurationMetaNames.DATA_HANDLER: {
+         >>>        "info": {
+         >>>            "npz_file": "./data_party0.npz"
+         >>>        },
+         >>>        "name": "MnistTFDataHandler",
+         >>>        "path": "./mnist_keras_data_handler.py"
+         >>>     },
+         >>>     wml_client.remote_training_systems.ConfigurationMetaNames.LOCAL_TRAINING: {
+         >>>        "name": "LocalTrainingHandler",
+         >>>        "path": "ibmfl.party.training.local_training_handler"
+         >>>     }
+         >>> }
+         >>> party = client.remote_training_systems.create_party(remote_training_system_id, party_metadata)
+
+        """
+
+        RemoteTrainingSystem._validate_type(remote_training_system_id, u'remote_training_system_id', STR_TYPE, True)
+        RemoteTrainingSystem._validate_type(party_metadata, u'party_metadata', dict, True)
+        self._validate_party_input(party_metadata)
+
+        host = self._client.wml_credentials['url'].split('//')[1]
+        party_config = {
+            "aggregator": {
+                "ip": host
+            },
+            "connection": {
+                "info": {
+                    "id": remote_training_system_id,
+                }
+            },
+            "data": party_metadata[self._client.remote_training_systems.ConfigurationMetaNames.DATA_HANDLER],
+            "protocol_handler": {
+                "name": "PartyProtocolHandler",
+                "path": "ibmfl.party.party_protocol_handler"
+            }
+        }
+
+        if "local_training" in party_metadata:
+            party_config["local_training"] = party_metadata[self._client.remote_training_systems.ConfigurationMetaNames.LOCAL_TRAINING]
+
+        return Party(client=self._client, config_dict=party_config)
\ No newline at end of file
