@@ -1,3 +1,17 @@
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import abc
 import traceback
 from os import environ
@@ -48,6 +62,7 @@ class AbstractTestAutoAIRemote(abc.ABC):
     run_id = None
 
     data_connection = None
+    test_data_connection = None
     results_connection = None
 
     train_data = None
@@ -78,6 +93,7 @@ class AbstractTestAutoAIRemote(abc.ABC):
     space_id = None
 
     asset_id = None
+    test_asset_id = None
     connection_id = None
 
     max_connection_nb = 5
@@ -141,10 +157,19 @@ class AbstractTestAutoAIRemote(abc.ABC):
         self.assertIsInstance(parameters, dict, msg='Config parameters are not a dictionary instance.')
 
     def test_05_fit_run_training_of_auto_ai_in_wml(self):
-        AbstractTestAutoAIRemote.trained_pipeline_details = self.remote_auto_pipelines.fit(
-            training_data_reference=[self.data_connection],
-            training_results_reference=self.results_connection,
-            background_mode=False)
+
+        # Do not pass test data connection if not set up (for example in CPD tests)
+        if self.test_data_connection:
+            AbstractTestAutoAIRemote.trained_pipeline_details = self.remote_auto_pipelines.fit(
+                training_data_reference=[self.data_connection],
+                test_data_references=[self.test_data_connection],
+                training_results_reference=self.results_connection,
+                background_mode=False)
+        else:
+            AbstractTestAutoAIRemote.trained_pipeline_details = self.remote_auto_pipelines.fit(
+                training_data_reference=[self.data_connection],
+                training_results_reference=self.results_connection,
+                background_mode=False)
 
         AbstractTestAutoAIRemote.run_id = self.trained_pipeline_details['metadata']['id']
         self.assertIsNotNone(self.data_connection.auto_pipeline_params,
