@@ -1,3 +1,17 @@
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import unittest
 import uuid
 
@@ -24,7 +38,7 @@ class TestAutoAIRemote(AbstractTestAutoAIAsync, AbstractTestWebservice, Abstract
     batch_payload_cos_location = 'scoring_payload/drug_train_data_updated_scoring_payload.csv'
 
 
-    SPACE_ONLY = False
+    SPACE_ONLY = True
 
     OPTIMIZER_NAME = "Drug data test sdk"
 
@@ -35,10 +49,6 @@ class TestAutoAIRemote(AbstractTestAutoAIAsync, AbstractTestWebservice, Abstract
     target_space_id = None
 
     fairness_info = {
-        # "protected_attributes": [
-        #     {"feature": "SEX", "privileged_groups": ['F']},
-        #     {"feature": "BP", "privileged_groups": ["LOW", "NORMAL"]}
-        # ],
         "favorable_labels": ["drugA", "drugC"]
     }
 
@@ -52,7 +62,8 @@ class TestAutoAIRemote(AbstractTestAutoAIAsync, AbstractTestWebservice, Abstract
                                  ClassificationAlgorithms.XGB],
         fairness_info=fairness_info,
         max_number_of_estimators=2,
-        text_processing=False
+        text_processing=False,
+        notebooks=True
     )
 
     def test_00b_prepare_COS_instance_and_connection(self):
@@ -89,13 +100,16 @@ class TestAutoAIRemote(AbstractTestAutoAIAsync, AbstractTestWebservice, Abstract
     def test_10_summary_listing_all_pipelines_from_wml(self):
         TestAutoAIRemote.summary = self.remote_auto_pipelines.summary()
         print(TestAutoAIRemote.summary)
-        self.assertIn('holdout_disparate_impact', list(TestAutoAIRemote.summary.columns))
-        self.assertIn('training_disparate_impact', list(TestAutoAIRemote.summary.columns))
-
 
         for col in self.summary.columns:
             print(self.summary[col])
 
+        self.assertIn('holdout_disparate_impact', list(TestAutoAIRemote.summary.columns))
+        self.assertIn('holdout_disparate_impact_SEX', list(TestAutoAIRemote.summary.columns))
+        self.assertIn('holdout_disparate_impact_AGE', list(TestAutoAIRemote.summary.columns))
+        self.assertIn('training_disparate_impact_AGE', list(TestAutoAIRemote.summary.columns))
+
+
     def test_99_delete_connection_and_connected_data_asset(self):
         if not self.SPACE_ONLY:
             self.wml_client.set.default_project(self.project_id)
