@@ -1,3 +1,17 @@
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import unittest
 
 import ibm_boto3
@@ -7,6 +21,7 @@ from ibm_watson_machine_learning.utils.autoai.errors import CannotReadSavedRemot
 from ibm_watson_machine_learning.tests.utils import bucket_exists, create_bucket, is_cp4d
 from ibm_watson_machine_learning.tests.autoai.abstract_tests_classes import (
     AbstractTestAutoAIRemote)
+from ibm_watson_machine_learning.utils.autoai.errors import NoAutomatedHoldoutSplit
 
 
 class TestAutoAIRemote(AbstractTestAutoAIRemote, unittest.TestCase):
@@ -44,6 +59,7 @@ class TestAutoAIRemote(AbstractTestAutoAIRemote, unittest.TestCase):
         )
 
     def test_00c_prepare_connection_to_COS(self):
+        self.wml_client.set.default_project(self.project_id)
         connection_details = self.wml_client.connections.create({
             'datasource_type': self.wml_client.connections.get_datasource_type_uid_by_name('bluemixcloudobjectstorage'),
             'name': 'Connection to COS for tests',
@@ -71,8 +87,21 @@ class TestAutoAIRemote(AbstractTestAutoAIRemote, unittest.TestCase):
         TestAutoAIRemote.asset_id = self.wml_client.data_assets.get_id(asset_details)
         self.assertIsInstance(self.asset_id, str)
 
+    def test_00e_prepare_connected_data_asset_for_test_result_references(self):
+        self.wml_client.set.default_project(self.project_id)
+
+        asset_details = self.wml_client.data_assets.store({
+            self.wml_client.data_assets.ConfigurationMetaNames.CONNECTION_ID: self.connection_id,
+            self.wml_client.data_assets.ConfigurationMetaNames.NAME: "Iris - test asset",
+            self.wml_client.data_assets.ConfigurationMetaNames.DATA_CONTENT_NAME: '/' + self.bucket_name + '/' + self.data_cos_path
+        })
+
+        TestAutoAIRemote.test_asset_id = self.wml_client.data_assets.get_id(asset_details)
+        self.assertIsInstance(self.test_asset_id, str)
+
     def test_02_DataConnection_setup(self):
         TestAutoAIRemote.data_connection = DataConnection(data_asset_id=self.asset_id)
+        TestAutoAIRemote.test_data_connection = DataConnection(data_asset_id=self.test_asset_id)
         TestAutoAIRemote.results_connection = DataConnection(
             connection_asset_id=self.connection_id,
             location=S3Location(
@@ -82,11 +111,32 @@ class TestAutoAIRemote(AbstractTestAutoAIRemote, unittest.TestCase):
         )
 
         self.assertIsNotNone(obj=TestAutoAIRemote.data_connection)
+        self.assertIsNotNone(obj=TestAutoAIRemote.test_data_connection)
         self.assertIsNotNone(obj=TestAutoAIRemote.results_connection)
 
     def test_02a_read_saved_remote_data_before_fit(self):
         self.assertRaises(NotImplementedError, self.data_connection.read)
 
+    def test_06b_get_train_data(self):  # test data
+        X_test, y_test = self.remote_auto_pipelines.get_test_data_connections()[0].read()
+
+        print("Test data sample:")
+        print(X_test)
+        print(y_test)
+
+        self.assertGreater(len(X_test), 0)
+        self.assertGreater(len(y_test), 0)
+
+    def test_06c_get_train_data_assert(self):
+        with self.assertRaises(NoAutomatedHoldoutSplit):
+            X_train, X_holdout, y_train, y_holdout = self.remote_auto_pipelines.get_data_connections()[0].read(
+                with_holdout_split=True)
+
+    def test_06d_get_test_data_assert(self):
+        with self.assertRaises(NoAutomatedHoldoutSplit):
+            X_test, y_test = self.remote_auto_pipelines.get_test_data_connections()[0].read(with_holdout_split=True)
+
+
     def test_29_delete_connection_and_connected_data_asset(self):
         self.wml_client.data_assets.delete(self.asset_id)
         self.wml_client.connections.delete(self.connection_id)
