@@ -1,3 +1,17 @@
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import unittest
 import uuid
 
@@ -20,6 +34,9 @@ class TestAutoAIRemote(AbstractTestAutoAIAsync, AbstractTestWebservice, Abstract
 
     data_cos_path = 'data/credit_risk_training_500.parquet'
 
+    batch_payload_location = './autoai/data/credit_risk_scoring_payload.csv'
+    batch_payload_cos_location = 'scoring_payload/credit_risk_scoring_payload.csv'
+
     SPACE_ONLY = False
 
     OPTIMIZER_NAME = "Credit Risk test sdk"
