@@ -1,3 +1,17 @@
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import unittest
 from sklearn.pipeline import Pipeline
 from pprint import pprint
@@ -427,7 +441,7 @@ class TestAutoAIRemote(unittest.TestCase):
                                  msg="Batch Deployment creation - model (asset) id missing, incorrect model storing")
 
     def test_32c_deploy__batch_deploy_best_computed_pipeline_from_autoai_on_wml(self):
-        pipeline_name = self.winning_pipelines_summary.reset_index()['Pipeline Name'][-1]
+        pipeline_name = self.winning_pipelines_summary.reset_index()['Pipeline Name'][0]
         print('Deploying', pipeline_name)
 
         self.service_batch.create(
