@@ -1,17 +1,3 @@
-# (C) Copyright IBM Corp. 2020.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 __all__ = [
     "MissingPipeline",
     "FitNotCompleted",
@@ -65,10 +51,29 @@ __all__ = [
     'TestDataNotPresent',
     'NoAutomatedHoldoutSplit',
     'DiscardedModel',
-    'WrongModelName'
+    'WrongModelName',
+    'StrategyIsNotApplicable',
+    'NumericalImputationStrategyValueMisused',
+    'InvalidImputationParameterNonTS',
+    'InvalidImputationParameterTS'
 ]
 
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 from ibm_watson_machine_learning.utils import WMLClientError
+from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, ImputationStrategy
 
 
 class MissingPipeline(WMLClientError, ValueError):
@@ -349,7 +354,7 @@ class NoAutomatedHoldoutSplit(WMLClientError, ValueError):
 class DiscardedModel(WMLClientError, ValueError):
     def __init__(self, model_name):
         WMLClientError.__init__(self,
-                                "You are trying to store a discarded TS model pipeline. Please use not discarded one."
+                                "You are trying to store a discarded forecasting model pipeline. Please use not discarded one."
                                 "Look at the pipelines 'summary' method and choose the 'Winner' one.",
                                 reason=f"Pipeline: '{model_name}' is discarded!")
 
@@ -360,3 +365,28 @@ class WrongModelName(WMLClientError, ValueError):
                                 "This model name does not exist. Please provide correct one."
                                 "Look at the pipelines 'summary' method and choose the 'Winner' one.",
                                 reason=f"Pipeline: '{model_name}' does not exist!")
+
+
+class StrategyIsNotApplicable(WMLClientError, ValueError):
+    def __init__(self, strategy, prediction_type, valid_strategies):
+        WMLClientError.__init__(self,
+                                f'{strategy} is not applicable for '
+                                f'{"forecasting" if prediction_type == PredictionType.FORECASTING else "non-forecasting"} '
+                                f'scenario. List of valid strategies is: {", ".join(valid_strategies)}')
+
+
+class NumericalImputationStrategyValueMisused(WMLClientError, ValueError):
+    def __init__(self):
+        WMLClientError.__init__(self, 'Parameter `numerical_imputation_value` can be only used when '
+                                      '`numerical_imputation_strategy` is set to ImputationStrategy.VALUE.')
+
+
+class InvalidImputationParameterNonTS(WMLClientError, ValueError):
+    def __init__(self):
+        WMLClientError.__init__(self, '`numerical_imputation_value` and `imputation_threshold` can be set only'
+                                      ' in forecasting scenario.')
+
+
+class InvalidImputationParameterTS(WMLClientError, ValueError):
+    def __init__(self):
+        WMLClientError.__init__(self, 'For forecasting scenario `categorical_imputation_strategy` should not be set.')
