@@ -1,17 +1,3 @@
-# (C) Copyright IBM Corp. 2020.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 __all__ = [
     'fetch_pipelines',
     'load_file_from_file_system',
@@ -27,6 +13,7 @@ __all__ = [
     'try_load_dataset',
     'check_dependencies_versions',
     'try_import_autoai_libs',
+    'try_import_autoai_ts_libs',
     'try_import_tqdm',
     'try_import_xlrd',
     'try_import_graphviz',
@@ -46,13 +33,29 @@ __all__ = [
     'is_list_composed_from_enum',
     'validate_optimizer_enum_values',
     'all_logging_disabled',
-    'check_if_ts_pipeline_is_winner'
+    'check_if_ts_pipeline_is_winner',
+    'get_values_for_imputation_strategy'
 ]
 
+#  (C) Copyright IBM Corp. 2021.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import io
 import json
 import os
 import enum
+import gzip
 import inspect
 from contextlib import redirect_stdout
 from functools import wraps
@@ -72,11 +75,11 @@ from packaging import version
 
 from .enums import (RegressionAlgorithms, RegressionAlgorithmsCP4D, ClassificationAlgorithms,
                     ClassificationAlgorithmsCP4D, ForecastingAlgorithms, ForecastingAlgorithmsCP4D, Transformers,
-                    Metrics, TShirtSize, PredictionType, DataConnectionTypes)
+                    Metrics, TShirtSize, PredictionType, DataConnectionTypes, ImputationStrategy)
 from .errors import (MissingPipeline, DataFormatNotSupported, LibraryNotCompatible,
                      CannotInstallLibrary, CannotDownloadTrainingDetails, CannotDownloadWMLPipelineDetails,
                      VisualizationFailed, AdditionalParameterIsUnexpected, InvalidSequenceValue, NoAvailableMetrics,
-                     DiscardedModel, WrongModelName)
+                     DiscardedModel, WrongModelName, StrategyIsNotApplicable)
 
 if TYPE_CHECKING:
     from io import BytesIO, BufferedIOBase
@@ -168,6 +171,7 @@ def fetch_pipelines(run_params: dict,
     pipelines_names = []
     pipelines = {}
     check_lale = True
+    is_ts_metrics = 'timeseries' in run_params['entity']['status'].get('metrics', [None])[0]['context']
 
     if wml_client.ICP:
         model_paths = []
@@ -231,9 +235,10 @@ def fetch_pipelines(run_params: dict,
         filenames = []
         keys = []
 
+        pipeline_suffix = "gz" if is_ts_metrics else "pickle"
         for pipeline in run_params['entity']['status'].get('metrics', []):
             model_number = pipeline['context']['intermediate_model']['name'].split('P')[-1]
-            model_phase = chose_model_output(model_number)
+            model_phase = chose_model_output(model_number, is_ts_metrics=is_ts_metrics)
 
             if pipeline['context']['phase'] == model_phase:
                 model_path = f"{pipeline['context']['intermediate_model']['location']['model']}"
@@ -241,7 +246,7 @@ def fetch_pipelines(run_params: dict,
                 if pipeline_name is None:
                     buckets.append(run_params['entity']['results_reference']['location']['bucket'])
                     filenames.append(
-                        f"{path}/Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}.pickle")
+                        f"{path}/Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}.{pipeline_suffix}")
                     keys.append(model_path)
                     pipelines_names.append(
                         f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}")
@@ -253,7 +258,7 @@ def fetch_pipelines(run_params: dict,
                 elif pipeline_name == f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}":
                     buckets = [run_params['entity']['results_reference']['location']['bucket']]
                     filenames = [
-                        f"{path}/Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}.pickle"]
+                        f"{path}/Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}.{pipeline_suffix}"]
                     keys = [model_path]
                     pipelines_names = [f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}"]
 
@@ -269,7 +274,12 @@ def fetch_pipelines(run_params: dict,
 
                 # Disable printing to suppress warning from ai4ml
                 with redirect_stdout(open(os.devnull, "w")):
-                    pipelines[name] = joblib.load(filename)
+                    if is_ts_metrics:
+                        with gzip.open(filename, "rb") as f:
+                            pipeline_content = f.read()
+                            pipelines[name] = joblib.load(io.BytesIO(pipeline_content))
+                    else:
+                        pipelines[name] = joblib.load(filename)
 
                 if not store:
                     if os.path.exists(filename):
@@ -281,7 +291,6 @@ def fetch_pipelines(run_params: dict,
                     # note: display download link to the model
                     create_model_download_link(filename)
                     # --- end note
-
     if load_pipelines and pipelines:
         return pipelines, check_lale
 
@@ -438,7 +447,9 @@ def preprocess_request_json(request_json: Dict) -> Dict:
 
 def chose_model_output(model_number: str, is_ml_metrics: bool = True, is_ts_metrics: bool = False) -> str:
     """Chose correct path for particular model number"""
-    if is_ml_metrics:
+    if is_ts_metrics:
+        return 'after_final_pipelines_generation'
+    elif is_ml_metrics:
         model_number = int(model_number)
         hpo_c_numbers = (4, 8, 12, 16)
         cognito_numbers = (3, 7, 11, 15)
@@ -460,9 +471,6 @@ def chose_model_output(model_number: str, is_ml_metrics: bool = True, is_ts_metr
         else:
             return 'global_output'
 
-    elif is_ts_metrics:
-        return 'after_pipeline_execution'
-
 
 def prepare_auto_ai_model_to_publish_notebook_normal_scenario(
         pipeline_model: Union['Pipeline', 'TrainablePipeline'],
@@ -976,6 +984,24 @@ def try_import_autoai_libs(minimum_version: str = None):
         install_autoai_libs(version=default_version)
 
 
+def try_import_autoai_ts_libs():
+    """
+    Check if autoai_ts_libs package is installed in local environment, if not, just download and install it.
+    """
+    try:
+        import autoai_ts_libs
+
+    except ImportError:
+        warn(f"\"autoai_libs\" package is not installed. "
+             f"This is the needed dependency for pipeline model refinery, we will try to install it now...")
+
+        try:
+            check_call([executable, "-m", "pip", "install", "autoai-ts-libs"])
+
+        except Exception as e:
+            raise CannotInstallLibrary(value_name=e,
+                                       reason="autoai-ts-libs failed to install. Please install it manually.")
+
 def try_import_tqdm():
     """
     Check if tqdm package is installed in local environment, if not, just download and install it.
@@ -1183,7 +1209,12 @@ def check_dependencies_versions(request_json: dict, wml_client, estimator_pkg: s
                 # workaround for autai-libs and numpy versions in SW spec
                 if package['name'] == 'autoai-libs' or package['name'] == 'numpy' \
                         or package['name'] == 'lale' or package['name'] == 'snapml':
-                    if version.parse(installed_module_version) < version.parse(package['version']):
+
+                    if package['name'] == 'snapml' and version.parse(installed_module_version) >= version.parse('1.8'):
+                        package['version'] = f"<1.8.0 and >={package['version']}"
+                        errored_packages.append(package)
+
+                    elif version.parse(installed_module_version) < version.parse(package['version']):
                         # we should produce different error for lale and snap_ml as only 2 major numbers are significant
                         packages_less_strict_error_list = ['lale', 'snapml']
                         if package['name'] in packages_less_strict_error_list and len(
@@ -1194,6 +1225,11 @@ def check_dependencies_versions(request_json: dict, wml_client, estimator_pkg: s
 
                         errored_packages.append(package)
 
+                # Note: temporary check / workaround for lightgbm on WS
+                elif package['name'] == 'lightgbm' or package['name'] == 'py-lightgbm':
+                    if installed_module_version == '3.1.1' or installed_module_version == '3.2.1':
+                        pass
+
                 else:
                     if installed_module_version != package['version']:
                         errored_packages.append(package)
@@ -1617,11 +1653,11 @@ def validate_additional_params_for_optimizer(params):
         'FE_drop_constant_columns_flag', 'FE_add_frequency_columns_flag', 'FE_add_missing_indicator_columns_flag',
         'data_provenance', 'target_label_name', 'preprocessor_data_filename', 'cognito_data_filename',
         'holdout_roc_curve_max_size', 'holdout_reg_pred_obs_max_size', 'max_estimator_n_jobs',
-        'enabled_feature_engineering_as_json', 'fairness_info', 'text_processing', 'word2vec_feature_number',
+        'enabled_feature_engineering_as_json',
         '_enable_snapml_estimators', 'return_holdout_indices', 'retrain_pipelines_on_holdout_data',
-        'retrain_on_holdout',
         'sampling_type', 'n_parallel_data_connections', 'calculate_data_metrics',
-        'batch_size', 'metrics_on_logical_batch', 'logical_batch_size', 'text_columns_names', 'enable_all_data_sources'
+        'batch_size', 'metrics_on_logical_batch', 'logical_batch_size', 'text_columns_names', 'enable_all_data_sources',
+        'scorer_fairness_weight'
     ]
 
     for k in params:
@@ -1732,6 +1768,7 @@ def validate_optimizer_enum_values(
                                             RegressionAlgorithms,
                                             ForecastingAlgorithms]],
         cognito_transform_names: List[Transformers],
+        imputation_strategies: List[ImputationStrategy],
         scoring: str,
         t_shirt_size: str,
         is_cpd=False) -> None:
@@ -1748,6 +1785,8 @@ def validate_optimizer_enum_values(
         List of estimators.
     cognito_transform_names: list
         List of transformers.
+    imputation_strategies: List
+        List of imputation strategies.
     scoring: str
         Type of the metric to optimize with.
     t_shirt_size: str
@@ -1778,16 +1817,10 @@ def validate_optimizer_enum_values(
             (daub_include_only_estimators, estimators_enum),
             (include_only_estimators, estimators_enum),
             (cognito_transform_names, Transformers),
-            ([t_shirt_size], TShirtSize)):
+            ([t_shirt_size], TShirtSize),
+            (imputation_strategies, ImputationStrategy)):
         is_list_composed_from_enum(sequence, enum_class)
 
-    # Note: scoring can be None or have one of values from Metrics enum.
-    # Note: allow pass fairness scoring values: TODO: Remove when official fairness support will be available
-    additional_allowed_scoring_values = ['r2_and_disparate_impact', 'accuracy_and_disparate_impact']
-    if scoring is not None:
-        if scoring not in additional_allowed_scoring_values:
-            is_list_composed_from_enum([scoring], Metrics)
-
 
 def fallback_to_pip_for_version_check(package: dict) -> str:
     """Use only when you need to check package version by package name with pip."""
@@ -1847,3 +1880,47 @@ def check_if_ts_pipeline_is_winner(details: dict, model_name: str) -> None:
         else:
             if not vals[0]:
                 raise DiscardedModel(model_name)
+
+
+def get_values_for_imputation_strategy(strategy, prediction_type):
+    values = {ImputationStrategy.MEAN: {'kb': 'mean',
+                                        'ts': {'use_imputation': True,
+                                               'imputer_list': ['Fill'],
+                                               'imputer_fill_type': 'mean'}},
+              ImputationStrategy.MEDIAN: {'kb': 'median',
+                                          'ts': {'use_imputation': True,
+                                                 'imputer_list': ['Fill'],
+                                                 'imputer_fill_type': 'median'}},
+              ImputationStrategy.MOST_FREQUENT: {'kb': 'most_frequent'},
+              ImputationStrategy.BEST_OF_DEFAULT_IMPUTERS: {'ts': {'use_imputation': True,
+                                                                   'imputer_list': ['FlattenIterative', 'Linear', 'Cubic', 'Previous']}},
+              ImputationStrategy.VALUE: {'ts': {'use_imputation': True,
+                                                'imputer_list': ['Fill'],
+                                                'imputer_fill_type': 'value',
+                                                'imputer_fill_value': 0}},
+              ImputationStrategy.FLATTEN_ITERATIVE: {'ts': {'use_imputation': True,
+                                                            'imputer_list': ['FlattenIterative']}},
+              ImputationStrategy.LINEAR: {'ts': {'use_imputation': True,
+                                                 'imputer_list': ['Linear']}},
+              ImputationStrategy.CUBIC: {'ts': {'use_imputation': True,
+                                                'imputer_list': ['Cubic']}},
+              ImputationStrategy.PREVIOUS: {'ts': {'use_imputation': True,
+                                                   'imputer_list': ['Previous']}},
+              ImputationStrategy.NEXT: {'ts': {'use_imputation': True,
+                                               'imputer_list': ['Next']}},
+              ImputationStrategy.NO_IMPUTATION: {'ts': {'use_imputation': False}}}
+
+    v = values[strategy]
+
+    if prediction_type == PredictionType.FORECASTING:
+        if 'ts' not in v:
+            l = ['ImputationStrategy.' + s.name for s in ImputationStrategy if values[s].get('ts')]
+            raise StrategyIsNotApplicable(strategy, prediction_type, l)
+
+        return v['ts']
+    else:
+        if 'kb' not in v:
+            l = ['ImputationStrategy.' + s.name for s in ImputationStrategy if values[s].get('kb')]
+            raise StrategyIsNotApplicable(strategy, prediction_type, l)
+
+        return v['kb']
