@@ -1,16 +1,16 @@
-# (C) Copyright IBM Corp. 2020.
+#  (C) Copyright IBM Corp. 2021.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#       http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
 
 from __future__ import print_function
 from ibm_watson_machine_learning.wml_client_error import WMLClientError
@@ -594,3 +594,22 @@ class NextResourceGenerator:
 
         else:
             raise StopIteration
+
+
+class DisableWarningsLogger:
+    """
+       Class which disables logging warnings (for example for silent handling WMLClientErrors in try except).
+
+        Example:
+        ----------
+        try:
+            with DisableWarningsLogger():
+                throw_wml_error()
+        except WMLClientError:
+            success = False
+    """
+    def __enter__(self):
+        logging.disable(logging.WARNING)
+
+    def __exit__(self, exit_type, exit_value, exit_traceback):
+        logging.disable(logging.NOTSET)
\ No newline at end of file
